// diffterrain.fx
// effect for drawing diffuse terrain map with detail layers and fog


///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////

texture		tDiffuseMap;		//      
texture		tLayer1;			// 1
texture		tLayer2;			// 2
texture		tBlendMap;			//     
texture		tShadowMap;			//  
// 
float4x4	matWorld;			//   ( identity)
float4x4	matViewProj;		//  View*Projection
float4x4	matLayer1;			//      1
float4x4	matLayer2;			//      2
float4x4	matLightViewProj;
float4x4	matShadowTexProj;
// 
float4		vParams;			// x - mindist (Fog=1), y - 1/(max-min), z - DepthBias, w - displace map scale
float3		vEyePos;			// xyz -  
float3		vLightDir;			//  
float4		vLightAmb;			// ambient 
float4		vLightDif;			// diffuse 
float		fLayerId;			//     


///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sDiffuseMapSampler=sampler_state
{
	Texture=(tDiffuseMap);
	AddressU=Clamp;
	AddressV=Clamp;
	MinFilter=Point;
	MipFilter=None;
	MagFilter=Point;
};

sampler sLayer1Sampler=sampler_state
{
	Texture=(tLayer1);
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sLayer2Sampler=sampler_state
{
	Texture=(tLayer2);
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sBlendMapSampler=sampler_state
{
	Texture=(tBlendMap);
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

sampler sShadowMapSampler=sampler_state
{
	Texture=(tShadowMap);
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};


///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float3 vPos		:POSITION;		//  ,    
	float3 vNormal	:NORMAL;		//  
	float2 vTex		:TEXCOORD0;		//    , 
									//        
};


///////////////////////////////////////////////////////////////////////////////////
// SHADOW MAP GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputShadowGen
{
	float4 vPos		:POSITION;		//  
	float fDepth	:TEXCOORD0;		//   
};

vsOutputShadowGen vsShadowGen(vsInput vIn)
{
	vsOutputShadowGen vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	vOut.vPos=mul(float4(vTPos,1),matLightViewProj);
	vOut.fDepth=vOut.vPos.z;
	return vOut;
}

float4 psShadowGen(vsOutputShadowGen vIn):COLOR0
{
	return vIn.fDepth;
}

technique ShadowGenTech
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=false;
		FillMode=(vParams.w);
		VertexShader=compile vs_2_0 vsShadowGen();
		PixelShader=compile ps_2_0 psShadowGen();
	}
}


///////////////////////////////////////////////////////////////////////////////////
// LAYERS BLENDING WITH DIFFUSE AND SHADOW MAP
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputBlended
{
	float4 vPos			:POSITION;	//  
	float4 vColor		:COLOR0;	//   xyz-color,w-fog
	float2 vDMTex		:TEXCOORD0;	//    
	float2 vL1Tex		:TEXCOORD1;	//   1
	float2 vL2Tex		:TEXCOORD2;	//   2
	float4 vTexShadow	:TEXCOORD3;	//    
	float fDepth		:TEXCOORD4;	//   
};


vsOutputBlended vsBlendLayers(vsInput vIn)
{
	vsOutputBlended vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	float fDist=distance(vEyePos,vTPos);
	float3 vTNormal=mul(vIn.vNormal,(float3x3)matWorld);
	float fDiffuse=clamp(dot(normalize(vTNormal),normalize(-vLightDir)),-0.3,1);
	vOut.vColor=float4((fDiffuse*vLightDif).xyz,1-saturate((fDist-vParams.x)*vParams.y));
	vOut.vPos=mul(float4(vTPos,1),matViewProj);
	vOut.vDMTex=vIn.vTex;
	vOut.vL1Tex=mul(float4(vIn.vPos.x,0,vIn.vPos.z,1),matLayer1).xz;
	vOut.vL2Tex=mul(float4(vIn.vPos.x,0,vIn.vPos.z,1),matLayer2).xz;
	vOut.vTexShadow=mul(float4(vTPos,1),matShadowTexProj);
	vOut.fDepth=mul(float4(vTPos,1),matLightViewProj).z;
	return vOut;
}

float4 psBlendLayers(vsOutputBlended vIn):COLOR0
{
	float2 DuDvCoords=vIn.vDMTex+tex2D(sBlendMapSampler,vIn.vDMTex)*vLayers.w;
	float4 DiffValue=tex2D(sDiffuseMapSampler,DuDvCoords);
	float4 LayersDelta=float4(DiffValue.w-vLayers.xy,0,0);
	LayersDelta=LayersDelta*LayersDelta*2048-0.5;
	float4 LayersCoeff;
	LayersCoeff=(LayersDelta<float4(0,0,0,0) ? float4(1,1,1,1) : float4(0,0,0,0));
	float4 Layer1=tex2D(sLayer1Sampler,vIn.vL1Tex)*LayersCoeff.x;
	float4 Layer2=tex2D(sLayer2Sampler,vIn.vL2Tex)*LayersCoeff.y;
	float LayersPresent=saturate(LayersCoeff.x+LayersCoeff.y);
	float4 LayersColor=(Layer1+Layer2)*(vIn.vColor.w);
	float4 TerrainColor=float4((DiffValue.xyz*(1-vIn.vColor.w)+LayersColor.xyz),LayersPresent);
	float fDepth=vIn.fDepth+vParams.z;
	float shdValue=((fDepth-tex2D(sShadowMapSampler,vIn.vTexShadow))>0 ? 0 : 1);
	return TerrainColor*((0.4+0.6*shdValue)*float4(vIn.vColor.xyz,1)+vLightAmb);
}

technique BlendLayersTech
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=(vParams.w);
		VertexShader=compile vs_2_0 vsBlendLayers();
		PixelShader=compile ps_2_0 psBlendLayers();
	}
}


///////////////////////////////////////////////////////////////////////////////////
// ONLY DIFFUSE AND SHADOW MAP
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputDiffuse
{
	float4 vPos			:POSITION;	//  
	float4 vColor		:COLOR0;	//   xyz-color,w-fog
	float2 vDMTex		:TEXCOORD0;	//   
	float4 vTexShadow	:TEXCOORD1;	//    
	float fDepth		:TEXCOORD2;	//   
};

vsOutputDiffuse vsDiffuse(vsInput vIn)
{
	vsOutputDiffuse vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	float3 vTNormal=mul(vIn.vNormal,(float3x3)matWorld);
	float fDiffuse=clamp(dot(normalize(vTNormal),normalize(-vLightDir)),-0.3,1);
	vOut.vColor=float4((fDiffuse*vLightDif).xyz,1);
	vOut.vPos=mul(float4(vTPos,1),matViewProj);
	vOut.vDMTex=vIn.vTex;
	vOut.vTexShadow=mul(float4(vTPos,1),matShadowTexProj);
	vOut.fDepth=mul(float4(vTPos,1),matLightViewProj).z;
	return vOut;
}

float4 psDiffuse(vsOutputBlended vIn):COLOR0
{
	float2 DuDvCoords=vIn.vDMTex+tex2D(sBlendMapSampler,vIn.vDMTex)*vLayers.w;
	float4 DiffValue=tex2D(sDiffuseMapSampler,DuDvCoords);
	DiffValue.w=1;
	float fDepth=vIn.fDepth+vParams.z;
	float shdValue=((fDepth-tex2D(sShadowMapSampler,vIn.vTexShadow))>0 ? 0 : 1);
	return DiffValue*(shdValue*float4(vIn.vColor.xyz,1)+vLightAmb);
}

technique DiffuseTech
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=false;
		FillMode=(vParams.w);
		VertexShader=compile vs_2_0 vsDiffuse();
		PixelShader=compile ps_2_0 psDiffuse();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// OLD VERSION
///////////////////////////////////////////////////////////////////////////////////

/*



VS_OUT VS_RGB(VS_IN vIn)
{
	VS_OUT vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	float fDist=distance(vEyePos,vTPos);
	float3 vTNormal=mul(vIn.vNormal,(float3x3)matWorld);
	float fDiffuse=clamp(dot(normalize(vTNormal),normalize(-vLightDir)),-0.3,1);
	vOut.vColor=float4((vLightAmb+fDiffuse*vLightDif).xyz,1);
	vOut.vColor.w=1-saturate((fDist-vFog.x)*vFog.y);
	vOut.vPos=mul(float4(vTPos,1),matViewProj);
	vOut.vDMTex=vIn.vTex;
	vOut.vL1Tex=mul(float4(vIn.vPos.x,0,vIn.vPos.z,1),matLayer1).xz;
	vOut.vL2Tex=mul(float4(vIn.vPos.x,0,vIn.vPos.z,1),matLayer2).xz;
	return vOut;
}

VS_OUT_DIFF VS_DIFF(VS_IN vIn)
{
	VS_OUT_DIFF vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	float3 vTNormal=mul(vIn.vNormal,(float3x3)matWorld);
	float fDiffuse=clamp(dot(normalize(vTNormal),normalize(-vLightDir)),-0.3,1);
	vOut.vColor=float4((vLightAmb+fDiffuse*vLightDif).xyz,1);
	vOut.vPos=mul(float4(vTPos,1),matViewProj);
	vOut.vDMTex=vIn.vTex;
	return vOut;
}

PixelShader PS_RGB=asm
{
	ps_2_0
	
	def c0,0,0,0,0
	def c1,1,1,1,1
	def c3,2048,-0.5,0,0
	dcl t0.xy
	dcl t1.xy
	dcl t2.xy
	dcl_2d s0
	dcl_2d s1
	dcl_2d s2
	dcl_2d s3
	dcl v0.rgba
	
	texld r1,t0,s1
	mad r1.xy,r1,c2.z,t0
	texld r2,t1,s2
	texld r3,t2,s3
	texld r0,r1,s0
	
	sub r4,r0.w,c2
	mul r4,r4,r4
	mad r4,r4,c3.x,c3.y
	mov r0.w,c1.x
	mov r8,c1
	cmp r4,r4,c0,r8
	mul r2.xyz,r2,r4.x				// 
	mov r2.w,r4.x
	mad r2.xyz,r3,r4.y,r2		// compute resulting layer color
	add_sat r2.w,r2.w,r4.y
	sub r8,c1.x,v0.w
	mul r0.xyz,r0,r8			// modulate diffuse color by fog
	mad r0.xyz,r2,v0.w,r0			// add layer color modulated by fog
	mul r0.xyz,r0,v0			// modulate by lighting color
	mov r0.w,r2.w
	mov oC0,r0
*//*
	ps_1_4
	// stage0 - diffuse map (r0)
	// stage1 - blend map (r1)
	// stage2 - layer1 (r2)
	// stage3 - layer2 (r3)
	// t0 - diffusemap and blend map coords
	// t1 - layer1 coords
	// t2 - layer2 coords
	// v0 - light color
	def c0,0,0,0,0
	def c1,1,1,1,1
	// def c2,Layer1,Layer2,du/dv Scale, 0	
//	def c3,2048,0,0,0
	
	texld r1,t0					// sample blend map
	texcrd r4.xyz,t0			// get t0 tex coords
	
	mad r1.xyz,r1,c2.z,r4		// distort diffusemap coords with scale

	phase
	
	texld r2,t1					// sample layer1
	texld r3,t2					// sample layer2
	
	texld r0,r1					// sample diffuse map using distorted coords
	sub_x8 r4.xy,r0.ww,c2		// compute layers delta (dL=(LayerID-UsedLayer)*8)
	+mov r0.w,c1.x
	mul_x8 r4.xy,r4_x2,r4_x2	// dL'=(dL*2*dL*2)*8=2048*dA^2
	cnd r4.xy,r4,c0,c1			// set r4.xy to 0 and 1 respectively layers delta
	mul r2.xyz,r2,r4.x			// 
	+mov r2.w,r4.x
	mad r2.xyz,r3,r4.y,r2		// compute resulting layer color
	+add_sat r2.w,r2.w,r4.y
	mul r0.xyz,r0,1-v0.w			// modulate diffuse color by fog
	mad r0.xyz,r2,v0.w,r0			// add layer color modulated by fog
	mul r0.xyz,r0,v0			// modulate by lighting color
	+mov r0.w,r2.w
*//*	
};

float4 psRGBHLSL(VS_OUT vIn):COLOR0
{
	float2 DuDvCoords=vIn.vDMTex+tex2D(sBlendMapSampler,vIn.vDMTex)*vLayers.z;
	float4 DiffValue=tex2D(sDiffuseMapSampler,DuDvCoords);
	float4 LayersDelta=float4(DiffValue.w-vLayers.xy,0,0);
	LayersDelta=LayersDelta*LayersDelta*2048-0.5;
	float4 LayersCoeff;
	LayersCoeff=(LayersDelta<float4(0,0,0,0) ? float4(1,1,1,1) : float4(0,0,0,0));
	float4 Layer1=tex2D(sLayer1Sampler,vIn.vL1Tex)*LayersCoeff.x;
	float4 Layer2=tex2D(sLayer2Sampler,vIn.vL2Tex)*LayersCoeff.y;
	float LayersPresent=saturate(LayersCoeff.x+LayersCoeff.y);
	float4 LayersColor=(Layer1+Layer2)*(vIn.vColor.w);
	return float4((DiffValue.xyz*(1-vIn.vColor.w)+LayersColor.xyz)*vIn.vColor.xyz,LayersPresent);
}

PixelShader PS_DIFF=asm
{
	ps_1_4
	// stage0 - diffuse map (r0)
	// stage1 - blend map (r1)
	// t0 - diffusemap and blend map coords
	// v0 - light color
	def c1,1,1,1,1
	
	texld r1,t0					// sample blend map
	texcrd r4.xyz,t0			// get t0 tex coords
	
	add r1.xyz,r4_bias,r1			// distort diffusemap coords
	
	phase
	
	texld r0,r1					// sample diffuse map using distorted coords
	mov r0.w,c1.x
};


technique RGBShading
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		Sampler[0]=(sDiffuseMapSampler);
		Sampler[1]=(sBlendMapSampler);
		Sampler[2]=(sLayer1Sampler);
		Sampler[3]=(sLayer2Sampler);
		FillMode=(nFillMode);
		PixelShaderConstant[2]=(vLayers);
		VertexShader=compile vs_2_0 VS_RGB();
//		PixelShader=(PS_RGB);
		PixelShader=compile ps_2_0 psRGBHLSL();
	}
}

technique DiffuseShading
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		Sampler[0]=(sDiffuseMapSampler);
		Sampler[1]=(sBlendMapSampler);
		FillMode=(nFillMode);
		VertexShader=compile vs_1_1 VS_DIFF();
		PixelShader=(PS_DIFF);
	}
}
*/