///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////

float4x4	matWorld;				//  
float4x4	matViewProj;			//  view*proj
float4x4	matLightViewProj;		//  view*proj
float4x4	matShadowTexProj;		//  view*proj*texel_adjustment
float3		vEyePos;				//  
float3		vLightDir;				//  
float4		cLightAmbient;			//  
float4		cLightDiffuse;			//  
float4		vTexDistortion;			//      1 (xy=uv)   2(zw=vu)
float4		vParams;				// x -  
									//  -    
texture		tWaterTex;				//       
texture		tGradientTex;			
texture		tBumpTex;				//   
texture		tShadowMap;				//  

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sWaterSampler=sampler_state
{
	Texture=(tWaterTex);
	AddressU=Wrap;
	AddressV=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sGradientSampler=sampler_state
{
	Texture=(tGradientTex);
	AddressU=Wrap;
	AddressV=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};


sampler sShadowSampler=sampler_state
{
	Texture=(tShadowMap);
	AddressU=Clamp;
	AddressV=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

sampler sBumpSampler=sampler_state
{
	Texture=(tBumpTex);
	AddressU=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};


///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float3	vPos		:POSITION;		//  
	float3	vNormal		:NORMAL;		// 
	float	fDepth		:TEXCOORD0;		//  
	float3  vTangent	:TEXCOORD1;		// 
};

///////////////////////////////////////////////////////////////////////////////////
// WATER SHADOW GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsShadowGenOutput
{
	float4	vPos:	POSITION;		//  
	float	fDepth:	TEXCOORD0;		//   
};

vsShadowGenOutput vsShadowGen(vsInput vIn)
{
	vsShadowGenOutput vOut;
//	vIn.vPos.y-=0.5;
	vOut.vPos=mul(float4(vIn.vPos,1),mul(matWorld,matLightViewProj));
	vOut.fDepth=vOut.vPos.z;
	return vOut;
}

float4 psShadowGen(vsShadowGenOutput vIn):COLOR0
{
	return vIn.fDepth;
}

technique TechShadowGen
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		VertexShader=compile vs_2_0 vsShadowGen();
		PixelShader=compile ps_2_0 psShadowGen();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// TEXTURE WATER + SHADOW OUTPUT SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsPlainWaterOutput
{
	float4	vPos:		POSITION;	//  
	float2	vTex:		TEXCOORD0;	//  
	float4  vDiffuse:	COLOR0;		//  
	float2	vShadowTex:	TEXCOORD1;	//  
	float2	vDepth:		TEXCOORD2;	// x -  , y -   
};

vsPlainWaterOutput vsPlainWaterShadow(vsInput vIn)
{
	vsPlainWaterOutput vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	vOut.vPos=mul(float4(vIn.vPos,1),matViewProj);
	vOut.vTex=vIn.vPos.xz*vParams.y+vTexDistortion*frac(vParams.x);	
	vOut.vDiffuse=cLightDiffuse*clamp(dot(float3(0,1,0),-vLightDir),-0.2,1);
	vOut.vShadowTex=mul(float4(vTPos,1),matShadowTexProj);
	vOut.vDepth=float2(vIn.fDepth,mul(float4(vTPos,1),matLightViewProj).z);
	return vOut;
}

float4 psPlainWaterShadow(vsPlainWaterOutput vIn):COLOR0
{
	float4 texColor=tex2D(sWaterSampler,vIn.vTex);
	float fDepth=vIn.vDepth.y-0.005;
	float shdValue=(fDepth-tex2D(sShadowSampler,vIn.vShadowTex)>0 ? 0: 1);
	float alpha=(vIn.vDepth.x>1 ? 0.7 : vIn.vDepth.x-0.3);
	return float4((texColor*(cLightAmbient+(0.4+0.6*shdValue)*vIn.vDiffuse)).xyz,alpha);
}

PixelShader psSimple=asm
{
	ps_1_4
	def c0,0.6,2,0.05,1
	texld r0,t0
	texcrd r1.xyz,t1
	add r2.x,r1.x,c0.z
	cnd r1.x,r1.x,c0.x,r2.x
	mul_sat r0.w,r1.x,c0.y
};


technique TechPlainWaterShadow
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaFunc=GreaterEqual;
		ZWriteEnable=false;
		VertexShader=compile vs_2_0 vsPlainWaterShadow();
		PixelShader=compile ps_2_0 psPlainWaterShadow();
//		Sampler[0]=(sWaterSampler);
//		PixelShader=(psSimple);
	}
}



///////////////////////////////////////////////////////////////////////////////////
// BUMP WATER + SHADOW OUTPUT SHADERS
///////////////////////////////////////////////////////////////////////////////////
struct vsBumpWaterOutput
{
	float4 vPos			:POSITION;		//  
	float4 vBumpTex		:TEXCOORD0;		// xy - 1 wz - 2
	float3 vView		:TEXCOORD1;		//     
	float3 vBinormal	:TEXCOORD2;		// 
	float3 vTangent		:TEXCOORD3;		// 
	float3 vNormal		:TEXCOORD4;		// 
	float2 vShadowTex	:TEXCOORD5;		//  
	float2 vDepth		:TEXCOORD6;		// x -  , y -   
//	float2 vPlainTex	:TEXCOORD7;
};

vsBumpWaterOutput vsBumpWaterShadow(vsInput vIn)
{
	vsBumpWaterOutput vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	vOut.vPos=mul(float4(vIn.vPos,1),matViewProj);
//	vOut.vPlainTex=(vIn.vPos.xz*vParams.y+frac(vTexDistortion*vParams.x));
	vOut.vBumpTex=(vIn.vPos.xzxz*vParams.z+frac(vTexDistortion*vParams.x));
	vOut.vTangent=vIn.vTangent;
	vOut.vNormal=vIn.vNormal;
	vOut.vBinormal=cross(vOut.vTangent,vOut.vNormal);
	vOut.vView=normalize(vEyePos-vTPos);
	vOut.vShadowTex=mul(float4(vTPos,1),matShadowTexProj);
	vOut.vDepth=float2(vIn.fDepth,mul(float4(vTPos,1),matLightViewProj).z);
	return vOut;
}

float4 psBumpWaterShadow(vsBumpWaterOutput vIn):COLOR0
{
	//  -
	float4 vBump1=tex2D(sBumpSampler,vIn.vBumpTex.xy);
	float4 vBump2=tex2D(sBumpSampler,vIn.vBumpTex.wz);
	float4 vOffset=(vBump1+vBump2)/4;
	float3 vNormal=normalize(vOffset.x*vIn.vBinormal+vOffset.y*vIn.vTangent+1*vIn.vNormal);
	//        
	float fViewCoeff=dot(vIn.vView,vNormal);
	float4 cWaterColor=tex1D(sGradientSampler,saturate(2*fViewCoeff));
	float fDepth=vIn.vDepth.y-0.005;
	float shdValue=(fDepth-tex2D(sShadowSampler,vIn.vShadowTex)>0 ? -1: 1);
	float3 vReflection=2*vNormal*fViewCoeff - vIn.vView;
	float4 cEnvColor=texCUBE(sWaterSampler,vReflection*shdValue);
	//    	
//	       	
	return cEnvColor*cEnvColor*(1-fViewCoeff);//+fViewCoeff*float4(0,0,1,1);
}


/*
struct vsBumpWaterOutput
{
	float4 vPos			:POSITION;		//  
	float4 vBumpTex		:TEXCOORD0;		// xy - 1 wz - 2
	float3 vView		:TEXCOORD1;		//     
	float3 vBinormal	:TEXCOORD2;		// 
	float3 vTangent		:TEXCOORD3;		// 
	float3 vNormal		:TEXCOORD4;		// 
	float2 vShadowTex	:TEXCOORD5;		//  
	float2 vDepth		:TEXCOORD6;		// x -  , y -   
	float2 vPlainTex	:TEXCOORD7;
};

vsBumpWaterOutput vsBumpWaterShadow(vsInput vIn)
{
	vsBumpWaterOutput vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	vOut.vPos=mul(float4(vIn.vPos,1),matViewProj);
	vOut.vPlainTex=(vIn.vPos.xz*vParams.y+frac(vTexDistortion*vParams.x));
	vOut.vBumpTex=(vIn.vPos.xzxz*vParams.z+frac(vTexDistortion*vParams.x));
	vOut.vTangent=vIn.vTangent;
	vOut.vNormal=vIn.vNormal;
	vOut.vBinormal=cross(vOut.vTangent,vOut.vNormal);
	vOut.vView=normalize(vEyePos-vTPos);
	vOut.vShadowTex=mul(float4(vTPos,1),matShadowTexProj);
	vOut.vDepth=float2(vIn.fDepth,mul(float4(vTPos,1),matLightViewProj).z);
	return vOut;
}



float4 psBumpWaterShadow(vsBumpWaterOutput vIn):COLOR0
{
	//  -
	float4 vBump1=tex2D(sBumpSampler,vIn.vBumpTex.xy);
	float4 vBump2=tex2D(sBumpSampler,vIn.vBumpTex.wz);
	float4 vOffset=(vBump1+vBump2)/4;
	float3 vNormal=normalize(vOffset.x*vIn.vBinormal+vOffset.y*vIn.vTangent+1*vIn.vNormal);
	//        
	float fViewCoeff=dot(vIn.vView,vNormal);
	float3 vReflection=2*vNormal*fViewCoeff - vIn.vView;
	float4 cWaterColor=tex1D(sGradientSampler,saturate(2*fViewCoeff));
	float4 cEnvColor=texCUBE(sWaterSampler,vReflection);
	//    	
	float fDiffuseCoeff=saturate(dot(-vLightDir,vNormal));
	//  
	float fDepth=vIn.vDepth.y-0.005;
	float shdValue=(fDepth-tex2D(sShadowSampler,vIn.vShadowTex)>0 ? 0: 1);
	//   
	float alpha=(vIn.vDepth.x>1 ? 0.7 : vIn.vDepth.x-0.3);
	//   
	float Ksat=saturate(dot(-vLightDir,vReflection));
	Ksat=Ksat*Ksat;
	Ksat=Ksat*Ksat;
	Ksat=Ksat*Ksat;
	Ksat=Ksat*Ksat;
	Ksat=Ksat*Ksat;
//	       	
//	return cEnvColor*cEnvColor*(1-fViewCoeff);
	
	return saturate((cEnvColor*cEnvColor*(1-fViewCoeff)+cWaterColor*2));
*/	
/*	float4 color=cEnvColor*cEnvColor;
	float s=cEnvColor.g;
	color*=1-fViewCoeff;
	color=saturate((color+cWaterColor*2)/4);
	color.xyz*=(cLightAmbient+(0.4+0.6*shdValue)*fDiffuseCoeff*cLightDiffuse).xyz;
	color.xyz=saturate(color.xyz+s*s*s*s*cLightDiffuse);
//	float4 tmp=saturate(cPlainColor*(1-fViewCoeff)+(fViewCoeff)*cWaterColor);
//	return float4((tmp*(cLightAmbient+(0.4+0.6*shdValue)*fDiffuseCoeff*cLightDiffuse)).xyz,1);//alpha);
	color.w=1;
	color=saturate(cEnvColor*(1-fViewCoeff)+cWaterColor+Ksat*cLightDiffuse);
	color.w=0.9;
	return color;*//*
}
*/
technique TechBumpWaterShadow
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaFunc=GreaterEqual;
//		ZWriteEnable=false;
		VertexShader=compile vs_2_0 vsBumpWaterShadow();
		PixelShader=compile ps_2_0 psBumpWaterShadow();
	}
}



/*
float4x4 mWVP;
float2 vDelta;
texture tWaterTex;

sampler sWaterSampler=sampler_state
{
	Texture=(tWaterTex);
	AddressU=Wrap;
	AddressV=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

struct VSO
{
	float4 vPos:POSITION;
	float2 vTex:TEXCOORD0;
	float  vDepth:TEXCOORD1;
};

struct VSI
{
	float3 vPos:POSITION;
	float vTex:TEXCOORD0;
};

VSO VS_Simple(VSI vIn)
{
	VSO vOut;
	vOut.vPos=mul(float4(vIn.vPos,1),mWVP);
	vOut.vTex=vIn.vPos.xz*0.05+vDelta;
	vOut.vDepth=vIn.vTex;
	return vOut;
}

PixelShader PS_Simple=asm
{
	ps_1_4
	def c0,0.6,2,0.05,1
	texld r0,t0
	texcrd r1.xyz,t1
	add r2.x,r1.x,c0.z
	cnd r1.x,r1.x,c0.x,r2.x
	mul_sat r0.w,r1.x,c0.y
};

technique SimpleWater
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		Sampler[0]=(sWaterSampler);
		AlphaFunc=GreaterEqual;
		VertexShader=compile vs_1_1 VS_Simple();
		PixelShader=(PS_Simple);
	}
}


*/
