// diffterrain.fx
// effect for drawing diffuse terrain map with detail layers and fog


///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////
#include "Inc/WarFog.fx"
#include "Inc/Shadows.fx"
#include "Inc/PointLight.fx"

texture		tDiffuseMap;		//      
texture		tBlendMap;			//     
texture		tLayerTex;			//  0
texture		tColorMap;
texture		tAlphaMask;
// 
float4x4	matViewProj;		//  View*Projection

float4x4	matLayer;			//      0
// 
float2		vParams;			// x - coords multiplicator, y - default blend radius
float		fDepthBias;

float3 vLightDir;			//  
float3 cLightAmbient;		// ambient 
float3 cLightDiffuse;		// diffuse 


///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sColorSampler = sampler_state
{
	Texture=(tColorMap);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

sampler sAlphaSampler = sampler_state
{
	Texture=(tAlphaMask);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};


sampler sLayerSampler=sampler_state
{
	Texture=(tLayerTex);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sBlendMapSampler=sampler_state
{
	Texture=(tBlendMap);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4 vPos		:POSITION;		//  ,    
	float3 vNormal	:NORMAL;		//  
	float2 vTex		:TEXCOORD0;		//    , 
									//        
};

/////////////////////////////////////////////////////////////////////////////////////////
// Shade terrain
/////////////////////////////////////////////////////////////////////////////////////////

struct vsOutputShade
{
	float4 vPos			:POSITION;	//  
	float4 vShadowParams:TEXCOORD0;
	float3 vFogPos		:TEXCOORD1;
};

vsOutputShade vsShadeTerrain(vsInput vIn)
{
	vsOutputShade vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
	return vOut;
}

float4 psShadeTerrain(vsOutputShade vIn):COLOR0
{
	return float4(0, 0, 0, 1 - Draw_GetShadowSimple(vIn.vShadowParams, fDepthBias) * GetWarFog(vIn.vFogPos));
}

technique TechShadeTerrain
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=Solid;
		AlphaTestEnable=true;
		AlphaRef=0x00;
		AlphaFunc=Greater;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=false;
		ZFunc = LessEqual;
		VertexShader=compile vs_2_0 vsShadeTerrain();
		PixelShader=compile ps_2_0 psShadeTerrain();
	}
}

/////////////////////////////////////////////////////////////////////////////////////////
// Defer light terrain
/////////////////////////////////////////////////////////////////////////////////////////

vsDeferLightOutput vsDeferLight(vsInput vIn)
{
	vsDeferLightOutput vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vTPos =vOut.vPos;
	vOut.vTNormal = vIn.vNormal;
	return vOut;
}

TECH_DEFER_LIGHT_DEFAULT

///////////////////////////////////////////////////////////////////////////////
// Layer Alpha blend
///////////////////////////////////////////////////////////////////////////////

struct vsOutputBlend
{
	float4 vPos			:POSITION;	//  
	float3 vColor		:COLOR0;	//   xyz-color
	float2 vAlphaTex	:TEXCOORD0;
	float2 vLayerTex	:TEXCOORD1;	//   
	float3 vWPos		:TEXCOORD2;
	float3 vWNormal		:TEXCOORD3;
};

vsOutputBlend vsBlendLayers(vsInput vIn)
{
	vsOutputBlend vOut;
	float4 vTPos = vIn.vPos;
	float4 vLPos = float4(vIn.vPos.x, 0, vIn.vPos.z, 1);
	float fDiffuse = max(dot(vIn.vNormal, -vLightDir), -0.3);
	vOut.vColor = fDiffuse * cLightDiffuse;
	vOut.vWPos = vIn.vPos;
	vOut.vWNormal = vIn.vNormal;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vAlphaTex = vIn.vTex;
	vOut.vLayerTex = mul(vLPos, matLayer).xz;
	return vOut;
}

float4 psBlendLayers(vsOutputBlend vIn):COLOR0
{
	float4 color = tex2D(sColorSampler, vIn.vAlphaTex);
	float2 DuDvCoords = vIn.vAlphaTex + tex2D(sBlendMapSampler, vIn.vAlphaTex * vParams.x) * vParams.y * color.w * 4;
	float alpha = tex2D(sAlphaSampler, DuDvCoords).x;
	float4 layer = tex2D(sLayerSampler, vIn.vLayerTex);
	float3 TerrainColor=saturate(layer * (color.xyz * 2));
	TerrainColor = TerrainColor * (vIn.vColor.xyz + cLightAmbient.xyz);
	return float4(TerrainColor, layer.w * alpha);
}

technique TechLayerBlend
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=Solid;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc = LessEqual;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsBlendLayers();
		PixelShader=compile ps_2_0 psBlendLayers();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Layer highlight
///////////////////////////////////////////////////////////////////////////////

struct vsOutputHighlight
{
	float4 vPos			:POSITION;	//  
	float2 vAlphaTex	:TEXCOORD0;
};

vsOutputHighlight vsHighlightLayer(vsInput vIn)
{
	vsOutputHighlight vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vAlphaTex = vIn.vTex;
	return vOut;
}

float4 psHighlightLayer(vsOutputHighlight vIn):COLOR0
{
	float alpha = tex2D(sAlphaSampler, vIn.vAlphaTex).x;
	return float4(1, 0, 0, alpha > 0 ? alpha * 0.5 + 0.5 : 0.1);
}

technique TechHighlightLayer
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=Solid;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZEnable=false;
		ZWriteEnable=false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsHighlightLayer();
		PixelShader=compile ps_2_0 psHighlightLayer();
	}
}




//EOF
